/**
 * Copyright (c) 2014-2016 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.registry.client.impl;

import static org.junit.Assert.assertEquals;

import org.junit.ClassRule;
import org.junit.Test;
import org.junit.rules.RuleChain;
import org.junit.rules.TestRule;
import org.ow2.petals.registry.client.api.PetalsRegistryClient;
import org.ow2.petals.registry.client.api.PetalsRegistryClientFactory;
import org.ow2.petals.registry.client.api.exception.ConnectionErrorException;
import org.ow2.petals.registry.client.api.exception.DuplicatedServiceException;
import org.ow2.petals.registry.client.api.exception.MissingServiceException;
import org.ow2.petals.registry_overlay.api.Constants;
import org.ow2.petals.registry_overlay.junit.PetalsRegistryOverlayNode;

/**
 * Unit test of {@link PetalsRegistryClientImpl} methods
 * 
 * @author Christophe DENEUX - Linagora
 * 
 */
public class PetalsRegistryClientFactoryImplTest {

    private static final PetalsRegistryOverlayNode CLUSTER_ON_DEFAULT_PORT = new PetalsRegistryOverlayNode(
            Constants.DEFAULT_PORT);

    private static final PetalsRegistryOverlayNode CLUSTER = new PetalsRegistryOverlayNode();

    // Caution to the rule ordering, the node listening on the default port must be instantiated first.
    @ClassRule
    public static final TestRule chain = RuleChain.outerRule(CLUSTER_ON_DEFAULT_PORT).around(CLUSTER);

    /**
     * <p>
     * Check the creation of a {@link PetalsRegistryClient} against:
     * <ul>
     * <li>all connection parameters are correct</li>
     * </ul>
     * </p>
     * <p>
     * Expected result:
     * <ul>
     * <li>no error occurs</li>
     * </ul>
     * </p>
     */
    @Test
    public void createPetalsRegistryClient_AllIsOk() throws ConnectionErrorException, DuplicatedServiceException,
            MissingServiceException {

        final PetalsRegistryClient registryClt = PetalsRegistryClientFactory.newInstance().createClient(
                CLUSTER.getHost(), CLUSTER.getPort(), CLUSTER.getGroupName(), CLUSTER.getPassword());
        try {
            assertEquals("Unexpected hostname", CLUSTER.getHost(), registryClt.getHost().getHostName());
            assertEquals("Unexpected port", CLUSTER.getPort(), registryClt.getPort());
            assertEquals("Unexpected user", CLUSTER.getGroupName(), registryClt.getGroup());
        } finally {
            registryClt.disconnect();
        }
    }

    /**
     * <p>
     * Check the creation of a {@link PetalsRegistryClient} against:
     * <ul>
     * <li>the remote host name is set to <code>null</code></li>
     * </ul>
     * </p>
     * <p>
     * Expected result:
     * <ul>
     * <li>no error occurs</li>
     * <li>the host used is <code>localhost</code></li>
     * </ul>
     * </p>
     */
    @Test
    public void createPetalsRegistryClient_NullHost() throws ConnectionErrorException, DuplicatedServiceException,
            MissingServiceException {

        final PetalsRegistryClient registryClt = PetalsRegistryClientFactory.newInstance().createClient(null,
                CLUSTER_ON_DEFAULT_PORT.getPort(), CLUSTER_ON_DEFAULT_PORT.getGroupName(),
                CLUSTER_ON_DEFAULT_PORT.getPassword());
        try {
            assertEquals("Unexpected hostname", CLUSTER_ON_DEFAULT_PORT.getHost(), registryClt.getHost().getHostName());
            assertEquals("Unexpected port", CLUSTER_ON_DEFAULT_PORT.getPort(), registryClt.getPort());
            assertEquals("Unexpected user", CLUSTER_ON_DEFAULT_PORT.getGroupName(), registryClt.getGroup());
        } finally {
            registryClt.disconnect();
        }
    }

    /**
     * <p>
     * Check the creation of a {@link PetalsRegistryClient} against:
     * <ul>
     * <li>the remote host port is set to <code>null</code></li>
     * </ul>
     * </p>
     * <p>
     * Expected result:
     * <ul>
     * <li>no error occurs</li>
     * <li>the host port used is the default one</li>
     * </ul>
     * </p>
     */
    @Test
    public void createPetalsRegistryClient_NullPort() throws ConnectionErrorException, DuplicatedServiceException,
            MissingServiceException {

        final PetalsRegistryClient registryClt = PetalsRegistryClientFactory.newInstance().createClient(
                CLUSTER_ON_DEFAULT_PORT.getHost(), null, CLUSTER_ON_DEFAULT_PORT.getGroupName(),
                CLUSTER_ON_DEFAULT_PORT.getPassword());
        try {
            assertEquals("Unexpected hostname", CLUSTER_ON_DEFAULT_PORT.getHost(), registryClt.getHost().getHostName());
            assertEquals("Unexpected port", CLUSTER_ON_DEFAULT_PORT.getPort(), registryClt.getPort());
            assertEquals("Unexpected group", CLUSTER_ON_DEFAULT_PORT.getGroupName(), registryClt.getGroup());
        } finally {
            registryClt.disconnect();
        }
    }

    /**
     * <p>
     * Check the creation of a {@link PetalsRegistryClient} against:
     * <ul>
     * <li>the group is set to <code>null</code></li>
     * <li>the server is protected by an empty credential group ("")</li>
     * </ul>
     * </p>
     * <p>
     * Expected result: the connection succeeds using the group ""
     * </p>
     */
    @Test
    public void createPetalsRegistryClient_DefaultGroup() throws ConnectionErrorException, DuplicatedServiceException,
            MissingServiceException {

        final PetalsRegistryClient registryClt = PetalsRegistryClientFactory.newInstance().createClient(
                CLUSTER.getHost(), CLUSTER.getPort(), null, CLUSTER.getPassword());
        try {
            assertEquals("Unexpected group", Constants.DEFAULT_CREDENTIALS_GROUP, registryClt.getGroup());
        } finally {
            registryClt.disconnect();
        }
    }

    /**
     * <p>
     * Check the creation of a {@link PetalsRegistryClient} against:
     * <ul>
     * <li>the group is set to <code>null</code></li>
     * <li>the server is protected by an empty credential password ("")</li>
     * </ul>
     * </p>
     * <p>
     * Expected result: the connection succeeds using the default password
     * {@value Constants#DEFAULT_CREDENTIALS_PASSWORD}.
     * </p>
     */
    @Test
    public void createPetalsRegistryClient_DefaultPassword() throws ConnectionErrorException,
            DuplicatedServiceException,
            MissingServiceException {

        final PetalsRegistryClient registryClt = PetalsRegistryClientFactory.newInstance().createClient(
                CLUSTER.getHost(), CLUSTER.getPort(), CLUSTER.getGroupName(), null);

        registryClt.disconnect();
    }

    /**
     * <p>
     * Check the creation of a {@link PetalsRegistryClient} against invalid credentials.
     * </p>
     * <p>
     * Expected result: the connection fails
     * </p>
     */
    @Test(expected = ConnectionErrorException.class)
    public void createPetalsRegistryClient_InvalidCredentails() throws ConnectionErrorException,
            DuplicatedServiceException, MissingServiceException {

        PetalsRegistryClientFactory.newInstance().createClient(CLUSTER_ON_DEFAULT_PORT.getHost(),
                CLUSTER_ON_DEFAULT_PORT.getPort(), "invalidGroup", CLUSTER_ON_DEFAULT_PORT.getPassword());
    }

}
